@@ -5,7 +5,7 @@
 
 var LibraryPThread = {
   $PThread__postset: 'if (!ENVIRONMENT_IS_PTHREAD) PThread.initMainThreadBlock();',
-  $PThread__deps: ['$PROCINFO', '_register_pthread_ptr', 'emscripten_main_thread_process_queued_calls', '$ERRNO_CODES'],
+  $PThread__deps: ['$PROCINFO', '_register_pthread_ptr', 'emscripten_main_thread_process_queued_calls', '$ERRNO_CODES', 'emscripten_futex_wake'],
   $PThread: {
     MAIN_THREAD_ID: 1, // A special constant that identifies the main JS thread ID.
     mainThreadInfo: {
@@ -313,8 +313,13 @@ var LibraryPThread = {
               }
             } else if (d.cmd === 'exitProcess') {
               // A pthread has requested to exit the whole application process (runtime).
-              Module['noExitRuntime'] = false;
-              exit(d.returnCode);
+              noExitRuntime = false;
+              try {
+                exit(d.returnCode);
+              } catch (e) {
+                if (e instanceof ExitStatus) return;
+                throw e;
+              }
             } else if (d.cmd === 'cancelDone') {
               PThread.returnWorkerToPool(worker);
             } else if (d.cmd === 'objectTransfer') {
@@ -507,8 +512,8 @@ var LibraryPThread = {
     {{{ makeSetValue('__num_logical_cores', 0, 'cores', 'i32') }}};
   },
 
-  {{{ USE_LSAN ? 'emscripten_builtin_' : '' }}}pthread_create__deps: ['_spawn_thread', 'pthread_getschedparam', 'pthread_self', 'memalign'],
-  {{{ USE_LSAN ? 'emscripten_builtin_' : '' }}}pthread_create: function(pthread_ptr, attr, start_routine, arg) {
+  {{{ USE_LSAN || USE_ASAN ? 'emscripten_builtin_' : '' }}}pthread_create__deps: ['_spawn_thread', 'pthread_getschedparam', 'pthread_self', 'memalign'],
+  {{{ USE_LSAN || USE_ASAN ? 'emscripten_builtin_' : '' }}}pthread_create: function(pthread_ptr, attr, start_routine, arg) {
     if (typeof SharedArrayBuffer === 'undefined') {
       err('Current environment does not support SharedArrayBuffer, pthreads are not available!');
       return {{{ cDefine('EAGAIN') }}};
@@ -719,7 +724,7 @@ var LibraryPThread = {
     if (canceled == 2) throw 'Canceled!';
   },
 
-  {{{ USE_LSAN ? 'emscripten_builtin_' : '' }}}pthread_join__deps: ['_cleanup_thread', '_pthread_testcancel_js', 'emscripten_main_thread_process_queued_calls'],
+  {{{ USE_LSAN ? 'emscripten_builtin_' : '' }}}pthread_join__deps: ['_cleanup_thread', '_pthread_testcancel_js', 'emscripten_main_thread_process_queued_calls', 'emscripten_futex_wait'],
   {{{ USE_LSAN ? 'emscripten_builtin_' : '' }}}pthread_join: function(thread, status) {
     if (!thread) {
       err('pthread_join attempted on a null thread pointer!');
@@ -1089,7 +1094,7 @@ var LibraryPThread = {
 
   __call_main: function(argc, argv) {
     var returnCode = _main(argc, argv);
-    if (!Module['noExitRuntime']) postMessage({ cmd: 'exitProcess', returnCode: returnCode });
+    if (!noExitRuntime) postMessage({ cmd: 'exitProcess', returnCode: returnCode });
     return returnCode;
   },
 
@@ -1152,40 +1157,38 @@ var LibraryPThread = {
     // all the args here.
     // We also pass 'sync' to C separately, since C needs to look at it.
     var numCallArgs = arguments.length - 2;
+#if ASSERTIONS
+    if (numCallArgs > {{{ cDefine('EM_QUEUED_JS_CALL_MAX_ARGS') }}}-1) throw 'emscripten_proxy_to_main_thread_js: Too many arguments ' + numCallArgs + ' to proxied function idx=' + index + ', maximum supported is ' + ({{{ cDefine('EM_QUEUED_JS_CALL_MAX_ARGS') }}}-1) + '!';
+#endif
     // Allocate a buffer, which will be copied by the C code.
     var stack = stackSave();
-    var buffer = stackAlloc(numCallArgs * 8);
+    // First passed parameter specifies the number of arguments to the function.
+    var args = stackAlloc(numCallArgs * 8);
+    var b = args >> 3;
     for (var i = 0; i < numCallArgs; i++) {
-      HEAPF64[(buffer >> 3) + i] = arguments[2 + i];
+      HEAPF64[b + i] = arguments[2 + i];
     }
-    var ret = _emscripten_run_in_main_runtime_thread_js(index, numCallArgs, buffer, sync);
+    var ret = _emscripten_run_in_main_runtime_thread_js(index, numCallArgs, args, sync);
     stackRestore(stack);
     return ret;
   },
 
-  emscripten_receive_on_main_thread_js__deps: ['emscripten_proxy_to_main_thread_js'],
-  emscripten_receive_on_main_thread_js: function(index, numCallArgs, buffer) {
-    // Avoid garbage by reusing a single JS array for call arguments.
-    if (!_emscripten_receive_on_main_thread_js.callArgs) {
-      _emscripten_receive_on_main_thread_js.callArgs = [];
-    }
-    var callArgs = _emscripten_receive_on_main_thread_js.callArgs;
-    callArgs.length = numCallArgs;
+  emscripten_receive_on_main_thread_js_callArgs: '=[]',
+
+  emscripten_receive_on_main_thread_js__deps: ['emscripten_proxy_to_main_thread_js', 'emscripten_receive_on_main_thread_js_callArgs'],
+  emscripten_receive_on_main_thread_js: function(index, numCallArgs, args) {
+    _emscripten_receive_on_main_thread_js_callArgs.length = numCallArgs;
+    var b = args >> 3;
     for (var i = 0; i < numCallArgs; i++) {
-      callArgs[i] = HEAPF64[(buffer >> 3) + i];
+      _emscripten_receive_on_main_thread_js_callArgs[i] = HEAPF64[b + i];
     }
     // Proxied JS library funcs are encoded as positive values, and
     // EM_ASMs as negative values (see include_asm_consts)
-    var func;
-    if (index > 0) {
-      func = proxiedFunctionTable[index];
-    } else {
-      func = ASM_CONSTS[-index - 1];
-    }
+    var func = index > 0 ? proxiedFunctionTable[index] : ASM_CONSTS[-index - 1];
 #if ASSERTIONS
     assert(func.length == numCallArgs);
 #endif
-    return func.apply(null, callArgs);
+    return func.apply(null, _emscripten_receive_on_main_thread_js_callArgs);
   },
 
 #if MODULARIZE
